from tesla_powerwall.responses import PowerwallStatus
import unittest

from tesla_powerwall import (
    Powerwall,
    GridStatus,
    SiteInfo,
    SiteMaster,
    MetersAggregates,
    Meter,
    MeterType,
    powerwall,
)

from tests.integration import POWERWALL_IP, POWERWALL_PASSWORD


class TestPowerwall(unittest.TestCase):
    def setUp(self) -> None:
        self.powerwall = Powerwall(POWERWALL_IP)
        self.powerwall.login(POWERWALL_PASSWORD)

    def tearDown(self) -> None:
        self.powerwall.close()

    def test_get_charge(self) -> None:
        charge = self.powerwall.get_charge()
        self.assertIsInstance(charge, float)

    def test_get_meters(self) -> None:
        meters = self.powerwall.get_meters()
        self.assertIsInstance(meters, MetersAggregates)

        self.assertIsInstance(meters.get_meter(MeterType.BATTERY), Meter)

        for meter_type in meters.meters:
            meter = meters.get_meter(meter_type)
            meter.energy_exported
            meter.energy_imported
            meter.instant_power
            meter.last_communication_time
            meter.frequency
            meter.average_voltage
            meter.get_energy_exported()
            meter.get_energy_imported()
            self.assertIsInstance(meter.get_power(), float)
            self.assertIsInstance(meter.is_active(), bool)
            self.assertIsInstance(meter.is_drawing_from(), bool)
            self.assertIsInstance(meter.is_sending_to(), bool)

    def test_sitemaster(self) -> None:
        sitemaster = self.powerwall.get_sitemaster()

        self.assertIsInstance(sitemaster, SiteMaster)

        sitemaster.status
        sitemaster.is_running
        sitemaster.is_connected_to_tesla
        sitemaster.is_power_supply_mode

    def test_site_info(self) -> None:
        site_info = self.powerwall.get_site_info()

        self.assertIsInstance(site_info, SiteInfo)

        site_info.nominal_system_energy
        site_info.site_name
        site_info.timezone

    def get_grid_status(self) -> None:
        grid_status = self.powerwall.get_grid_status()
        self.assertIsInstance(grid_status, GridStatus)

    def get_status(self) -> None:
        status = self.powerwall.get_status()
        self.assertIsInstance(status, PowerwallStatus)
        status.up_time_seconds
        status.start_time
        status.version



from typing import Union, List
import requests
from distutils import version

from .api import API
from .const import (
    DEFAULT_KW_ROUND_PERSICION,
    SUPPORTED_OPERATION_MODES,
    DeviceType,
    GridState,
    GridStatus,
    LineStatus,
    MeterType,
    OperationMode,
    Roles,
    SyncType,
    User,
)
from .responses import (
    LoginResponse,
    Meter,
    MetersAggregates,
    PowerwallStatus,
    SiteMaster,
    SiteInfo,
    Solar,
)
from .helpers import assert_attribute


class Powerwall:
    def __init__(
        self,
        endpoint: str,
        timeout: int = 10,
        http_session: requests.Session = None,
        verify_ssl: bool = False,
        disable_insecure_warning: bool = True,
        pin_version: Union[str, version.Version] = None,
    ):
        if pin_version is not None:
            self.pin_version(pin_version)
        else:
            self._pin_version = None

        self._api = API(
            endpoint,
            timeout,
            http_session,
            verify_ssl,
            disable_insecure_warning,
        )

    def login_as(
        self,
        user: Union[User, str],
        password: str,
        email: str,
        force_sm_off: bool = False,
    ) -> dict:
        if isinstance(user, User):
            user = user.value

        response = self._api.login(user, email, password, force_sm_off)
        # The api returns an auth cookie which is automatically set
        # so there is no need to further process the response

        return LoginResponse(response)

    def login(self, password: str, email: str = "", force_sm_off: bool = False) -> dict:
        return self.login_as(User.CUSTOMER, password, email, force_sm_off)

    def logout(self):
        self._api.logout()

    def is_authenticated(self) -> bool:
        return self._api.is_authenticated()

    def run(self):
        self._api.get_sitemaster_run()

    def stop(self):
        self._api.get_sitemaster_stop()

    def get_charge(self) -> float:
        return assert_attribute(self._api.get_system_status_soe(), "percentage", "soe")

    def get_sitemaster(self) -> SiteMaster:
        return SiteMaster(self._api.get_sitemaster())

    def get_meters(self) -> MetersAggregates:
        return MetersAggregates(self._api.get_meters_aggregates())

    def get_grid_status(self) -> GridStatus:
        """Returns the current grid status."""
        status = assert_attribute(
            self._api.get_system_status_grid_status(), "grid_status", "grid_status"
        )

        return GridStatus(status)

    def is_grid_services_active(self) -> bool:
        return assert_attribute(
            self._api.get_system_status_grid_status(),
            "grid_services_active",
            "grid_status",
        )

    def get_site_info(self) -> SiteInfo:
        """Returns information about the powerwall site"""
        return SiteInfo(self._api.get_site_info())

    def set_site_name(self, site_name: str):
        return self._api.post_site_info_site_name({"site_name": site_name})

    def get_status(self) -> PowerwallStatus:
        return PowerwallStatus(self._api.get_status())

    def get_device_type(self) -> DeviceType:
        """Returns the device type of the powerwall"""
        if self._pin_version is None or self._pin_version >= version.LooseVersion(
            "1.46.0"
        ):
            return self.get_status().device_type
        else:
            return DeviceType(
                assert_attribute(
                    self._api.get_device_type(), "device_type", "device_type"
                )
            )

    def get_serial_numbers(self) -> List[str]:
        powerwalls = assert_attribute(
            self._api.get_powerwalls(), "powerwalls", "powerwalls"
        )
        return [
            assert_attribute(powerwall, "PackageSerialNumber")
            for powerwall in powerwalls
        ]

    def get_operation_mode(self) -> OperationMode:
        operation_mode = assert_attribute(
            self._api.get_operation(), "real_mode", "operation"
        )
        return OperationMode(operation_mode)

    def get_backup_reserve_percentage(self) -> float:
        return assert_attribute(
            self._api.get_operation(), "backup_reserve_percent", "operation"
        )

    def get_solars(self) -> List[Solar]:
        return [Solar(solar) for solar in self._api.get_solars()]

    def get_vin(self) -> str:
        return assert_attribute(self._api.get_config(), "vin", "config")

    def get_version(self) -> str:
        return assert_attribute(self._api.get_status(), "version", "status")

    def detect_and_pin_version(self) -> str:
        self.pin_version(self.get_version())
        return self._pin_version

    def pin_version(self, vers: Union[str, version.Version]):
        if isinstance(vers, version.Version):
            self._pin_version = vers
        else:
            self._pin_version = version.LooseVersion(vers)

    def get_pinned_version(self) -> version.Version:
        return self._pin_version

    def get_api(self):
        return self._api

    def close(self):
        self._api.close()



from .api import API
from .const import (
    DEFAULT_KW_ROUND_PERSICION,
    SUPPORTED_OPERATION_MODES,
    DeviceType,
    GridState,
    GridStatus,
    LineStatus,
    MeterType,
    OperationMode,
    Roles,
    SyncType,
    User,
)
from .error import (
    AccessDeniedError,
    APIError,
    MissingAttributeError,
    PowerwallError,
    PowerwallUnreachableError,
)
from .helpers import assert_attribute, convert_to_kw
from .responses import (
    LoginResponse,
    Meter,
    MetersAggregates,
    PowerwallStatus,
    SiteInfo,
    SiteMaster,
    Solar,
)
from .powerwall import Powerwall

VERSION = "0.3.12"



# This is an Example of how to use the tesla_powerwall API and the influxdb Client to generate and store
# Monitoring Data in a Time Series Database. InfluxDB is natively compatible with https://grafana.com

#Imports
import os
from time import sleep
import influxdb_client
from influxdb_client.client.write_api import SYNCHRONOUS
from tesla_powerwall import Powerwall


# Variables
## InfluxDB
bucket = "<my-bucket>"
org = "<my-org>"
token = "<my-token>"
url="http://localhost:8086"

client = influxdb_client.InfluxDBClient(
    url=url,
    token=token,
    org=org
)

write_api = client.write_api(write_options=SYNCHRONOUS)

## Powerwall

ip = os.getenv("POWERWALL_IP")
if ip is None:
    raise ValueError("POWERWALL_IP must be set")

email = os.getenv("POWERWALL_EMAIL")
password = os.getenv("POWERWALL_PASSWORD")

power_wall = Powerwall(ip)

# Program

## Identify the powerwall version
power_wall.detect_and_pin_version()
print("Detected and pinned version: {}".format(power_wall.get_pinned_version()))

print("Current charge: {}".format(power_wall.get_charge()))
print("Device Type: {}".format(power_wall.get_device_type()))

## Sending Data

while True:
    p = influxdb_client.Point("Measurement").field("Charge", power_wall.get_charge)
    write_api.write(bucket=bucket, org=org, record = p)
    sleep(1)



import os
from tesla_powerwall import Powerwall

def getenv(var):
    val = os.getenv(var)
    if val is None:
        raise ValueError(f"{var} must be set")
    return val

ip = getenv("POWERWALL_IP")
password = getenv("POWERWALL_PASSWORD")

power_wall = Powerwall(ip)
power_wall.login(password)

# Identify the powerwall version
power_wall.detect_and_pin_version()
print("Detected and pinned version: {}".format(power_wall.get_pinned_version()))

print("Current charge: {}".format(power_wall.get_charge()))
print("Device Type: {}".format(power_wall.get_device_type()))
print("Site Name: {}".format(power_wall.get_site_info().site_name))


import os
import json

ENDPOINT = "https://1.1.1.1/api/"

PREFIX = "tests/unit/fixtures"


def _load_fixtures():
    for file in os.listdir("tests/unit/fixtures"):
        if file.endswith(".json"):
            file_path = os.path.join(PREFIX, file)
            with open(file_path) as f:
                name = file[:-5].upper() + "_RESPONSE"
                globals()[name] = json.loads(f.read())


_load_fixtures()



import re
from datetime import datetime, timedelta

from .const import (
    DEFAULT_KW_ROUND_PERSICION,
    DeviceType,
    MeterType,
    Roles,
)
from .helpers import assert_attribute, convert_to_kw


class Response:
    def __init__(self, response: dict):
        self.response = response

    def assert_attribute(self, attr: str):
        return assert_attribute(self.response, attr)

    def __repr__(self):
        return str(self.response)


class Meter(Response):
    """
    Attributes:
    - last_communication_time
    - instant_power
    - instant_reactive_power
    - instant_apparent_power
    - frequency
    - energy_exported
    - energy_imported
    - instant_average_voltage
    - instant_total_current
    - i_a_current
    - i_b_current
    - i_c_current
    - timeout
    """

    def __init__(self, meter: MeterType, response):
        self.meter = meter
        super().__init__(response)

    @property
    def instant_power(self):
        return self.assert_attribute("instant_power")

    @property
    def last_communication_time(self):
        return self.assert_attribute("last_communication_time")

    @property
    def frequency(self):
        return self.assert_attribute("frequency")

    @property
    def energy_exported(self):
        return self.assert_attribute("energy_exported")

    def get_energy_exported(self, precision=DEFAULT_KW_ROUND_PERSICION) -> float:
        return convert_to_kw(self.energy_exported, precision)

    @property
    def energy_imported(self):
        return self.assert_attribute("energy_imported")

    def get_energy_imported(self, precision=DEFAULT_KW_ROUND_PERSICION) -> float:
        return convert_to_kw(self.energy_imported, precision)

    @property
    def instant_total_current(self):
        return self.assert_attribute("instant_total_current")

    def get_instant_total_current(self, precision=DEFAULT_KW_ROUND_PERSICION) -> float:
        return round(self.instant_total_current, precision)

    @property
    def average_voltage(self):
        return self.assert_attribute("instant_average_voltage")

    def get_power(self, precision=DEFAULT_KW_ROUND_PERSICION) -> float:
        return convert_to_kw(self.instant_power, precision)

    def is_active(self, precision=DEFAULT_KW_ROUND_PERSICION) -> bool:
        return self.get_power(precision) != 0

    def is_drawing_from(self, precision=DEFAULT_KW_ROUND_PERSICION) -> bool:
        if self.meter == MeterType.LOAD:
            # Cannot draw from load
            return False
        else:
            return self.get_power(precision) > 0

    def is_sending_to(self, precision=DEFAULT_KW_ROUND_PERSICION) -> bool:
        if self.meter == MeterType.LOAD:
            # For load the power is always positiv
            return self.get_power(precision) > 0
        else:
            return self.get_power(precision) < 0


class MetersAggregates(Response):
    def __init__(self, response):
        super().__init__(response)
        self.meters = [MeterType(key) for key in response.keys()]

    def get_meter(self, meter: MeterType) -> Meter:
        if meter in self.meters:
            return Meter(meter, self.assert_attribute(meter.value))
        else:
            return None


class SiteMaster(Response):
    """
    Attributes:
    - running
    - connected_to_tesla
    - status
    - power_supply_mode
    """

    def __init__(self, response):
        super().__init__(response)

    @property
    def status(self):
        return self.assert_attribute("status")

    @property
    def is_running(self) -> bool:
        return self.assert_attribute("running")

    @property
    def is_connected_to_tesla(self) -> bool:
        return self.assert_attribute("connected_to_tesla")

    @property
    def is_power_supply_mode(self) -> bool:
        return self.assert_attribute("power_supply_mode")


class SiteInfo(Response):
    """
    Attributes:
    - max_site_meter_power_kW
    - min_site_meter_power_kW
    - nominal_system_energy_kWh
    - nominal_system_power_kW
    - max_system_energy_kWh
    - max_system_power_kW
    - site_name
    - timezone
    - grid_code
    """

    def __init__(self, response):
        super().__init__(response)

    @property
    def nominal_system_energy(self):
        return self.assert_attribute("nominal_system_energy_kWh")

    @property
    def site_name(self):
        return self.assert_attribute("site_name")

    @property
    def timezone(self):
        return self.assert_attribute("timezone")


class PowerwallStatus(Response):
    """
    Attributes:
    * start_time
    * up_time_seconds
    * is_new
    * version
    * device_type
    * commission_count
    * sync_type
    * git_hash
    """

    _START_TIME_FORMAT = "%Y-%m-%d %H:%M:%S %z"
    _UP_TIME_SECONDS_REGEX = re.compile(
        r"^((?P<days>[\.\d]+?)d)?((?P<hours>[\.\d]+?)h)?((?P<minutes>[\.\d]+?)m)?((?P<seconds>[\.\d]+?)s)?$"
    )

    def _parse_uptime_seconds(self, up_time_seconds: str):
        match = PowerwallStatus._UP_TIME_SECONDS_REGEX.match(up_time_seconds)
        if not match:
            raise ValueError(
                "Unable to parse up time seconds {}".format(up_time_seconds)
            )

        time_params = {}
        for (name, param) in match.groupdict().items():
            if param:
                time_params[name] = float(param)

        return timedelta(**time_params)

    @property
    def up_time_seconds(self):
        up_time_seconds = assert_attribute(self.response, "up_time_seconds")
        return self._parse_uptime_seconds(up_time_seconds)

    @property
    def start_time(self):
        start_time = assert_attribute(self.response, "start_time")
        return datetime.strptime(start_time, self._START_TIME_FORMAT)

    @property
    def version(self):
        return self.assert_attribute("version")

    @property
    def device_type(self):
        return DeviceType(self.assert_attribute("device_type"))


class LoginResponse(Response):
    """
    Attributes
    - email
    - firstname
    - lastname
    - roles
    - token
    - provider
    - loginTime
    """

    @property
    def firstname(self):
        return self.assert_attribute("firstname")

    @property
    def lastname(self):
        return self.assert_attribute("lastname")

    @property
    def token(self):
        return self.assert_attribute("token")

    @property
    def roles(self):
        return [Roles(role) for role in self.assert_attribute("roles")]

    @property
    def login_time(self):
        return self.assert_attribute("login_time")


class Solar(Response):
    """
    Attributes
    - brand
    - model
    - power_rating_watts
    """

    @property
    def brand(self):
        return self.assert_attribute("brand")

    @property
    def model(self):
        return self.assert_attribute("model")

    @property
    def power_rating_watts(self):
        return self.assert_attribute("power_rating_watts")



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="tesla_powerwall",
    author="Jrester",
    author_email="jrester379@gmail.com",
    version='0.3.12',
    description="API for Tesla Powerwall",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jrester/tesla_powerwall",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ],
    install_requires=["requests>=2.22.0"],
)



import json
import os
import unittest
import datetime

from distutils import version
import requests
import responses
from responses import GET, POST, Response, add

from tesla_powerwall import (
    API,
    AccessDeniedError,
    APIError,
    Meter,
    MetersAggregates,
    Powerwall,
    PowerwallUnreachableError,
    SiteMaster,
    SiteInfo,
    GridStatus,
    DeviceType,
    assert_attribute,
    convert_to_kw,
    MissingAttributeError,
    MeterType,
)


from tests.unit import (
    ENDPOINT,
    METERS_AGGREGATES_RESPONSE,
    STATUS_RESPONSE,
    GRID_STATUS_RESPONSE,
    SITE_INFO_RESPONSE,
    POWERWALLS_RESPONSE,
    SITEMASTER_RESPONSE,
    OPERATION_RESPONSE,
)


class TestPowerWall(unittest.TestCase):
    def setUp(self):
        self.powerwall = Powerwall(ENDPOINT)

    def test_get_api(self):
        self.assertIsInstance(self.powerwall.get_api(), API)

    def test_pins_version_on_creation(self):
        pw = Powerwall(ENDPOINT, pin_version="1.49.0")
        self.assertEqual(pw.get_pinned_version(), version.LooseVersion("1.49.0"))

        pw = Powerwall(ENDPOINT, pin_version=version.LooseVersion("1.49.0"))
        self.assertEqual(pw.get_pinned_version(), version.LooseVersion("1.49.0"))

    @responses.activate
    def test_get_charge(self):
        add(
            Response(
                GET, url=f"{ENDPOINT}system_status/soe", json={"percentage": 53.123423}
            )
        )
        self.assertEqual(self.powerwall.get_charge(), 53.123423)

    @responses.activate
    def test_get_sitemaster(self):
        add(
            Response(
                responses.GET, url=f"{ENDPOINT}sitemaster", json=SITEMASTER_RESPONSE
            )
        )

        sitemaster = self.powerwall.get_sitemaster()
        self.assertIsInstance(sitemaster, SiteMaster)

        self.assertEqual(sitemaster.status, "StatusUp")
        self.assertEqual(sitemaster.is_running, True)
        self.assertEqual(sitemaster.is_connected_to_tesla, True)
        self.assertEqual(sitemaster.is_power_supply_mode, False)

    @responses.activate
    def test_get_meters(self):
        add(
            Response(
                responses.GET,
                url=f"{ENDPOINT}meters/aggregates",
                json=METERS_AGGREGATES_RESPONSE,
            )
        )
        meters = self.powerwall.get_meters()
        self.assertIsInstance(meters, MetersAggregates)
        self.assertListEqual(
            meters.meters,
            [MeterType.SITE, MeterType.BATTERY, MeterType.LOAD, MeterType.SOLAR],
        )
        self.assertIsInstance(meters.get_meter(MeterType.LOAD), Meter)

    @responses.activate
    def test_meter(self):
        add(
            Response(
                responses.GET,
                url=f"{ENDPOINT}meters/aggregates",
                json=METERS_AGGREGATES_RESPONSE,
            )
        )

    @responses.activate
    def test_is_sending(self):
        add(
            Response(
                responses.GET,
                url=f"{ENDPOINT}meters/aggregates",
                json=METERS_AGGREGATES_RESPONSE,
            )
        )
        meters = self.powerwall.get_meters()
        self.assertEqual(meters.get_meter(MeterType.SOLAR).is_sending_to(), False)
        self.assertEqual(meters.get_meter(MeterType.SOLAR).is_active(), True)
        self.assertEqual(meters.get_meter(MeterType.SOLAR).is_drawing_from(), True)
        self.assertEqual(meters.get_meter(MeterType.SITE).is_sending_to(), True)
        self.assertEqual(meters.get_meter(MeterType.LOAD).is_sending_to(), True)
        self.assertEqual(meters.get_meter(MeterType.LOAD).is_drawing_from(), False)
        self.assertEqual(meters.get_meter(MeterType.LOAD).is_active(), True)

    @responses.activate
    def test_get_grid_status(self):
        add(
            Response(
                responses.GET,
                url=f"{ENDPOINT}system_status/grid_status",
                json=GRID_STATUS_RESPONSE,
            )
        )
        grid_status = self.powerwall.get_grid_status()
        self.assertEqual(grid_status, GridStatus.CONNECTED)

    @responses.activate
    def test_is_grid_services_active(self):
        add(
            Response(
                responses.GET,
                url=f"{ENDPOINT}system_status/grid_status",
                json=GRID_STATUS_RESPONSE,
            )
        )
        self.assertEqual(self.powerwall.is_grid_services_active(), False)

    @responses.activate
    def test_get_site_info(self):
        add(
            Response(responses.GET, url=f"{ENDPOINT}site_info", json=SITE_INFO_RESPONSE)
        )
        site_info = self.powerwall.get_site_info()
        self.assertEqual(site_info.nominal_system_energy, 27)
        self.assertEqual(site_info.site_name, "test")
        self.assertEqual(site_info.timezone, "Europe/Berlin")

    @responses.activate
    def test_get_status(self):
        add(Response(responses.GET, url=f"{ENDPOINT}status", json=STATUS_RESPONSE))
        status = self.powerwall.get_status()
        self.assertEqual(
            status.up_time_seconds,
            datetime.timedelta(seconds=61891, microseconds=214751),
        )
        self.assertEqual(
            status.start_time,
            datetime.datetime(
                2020,
                10,
                28,
                20,
                14,
                11,
                tzinfo=datetime.timezone(datetime.timedelta(seconds=28800)),
            ),
        )
        self.assertEqual(status.device_type, DeviceType.GW1)
        self.assertEqual(status.version, "1.50.1")

    @responses.activate
    def test_get_device_type(self):
        add(Response(responses.GET, url=f"{ENDPOINT}status", json=STATUS_RESPONSE))
        device_type = self.powerwall.get_device_type()
        self.assertIsInstance(device_type, DeviceType)
        self.assertEqual(device_type, DeviceType.GW1)

    @responses.activate
    def test_get_serial_numbers(self):
        add(
            Response(
                responses.GET, url=f"{ENDPOINT}powerwalls", json=POWERWALLS_RESPONSE
            )
        )
        serial_numbers = self.powerwall.get_serial_numbers()
        self.assertEqual(serial_numbers, ["SerialNumber1", "SerialNumber2"])

    @responses.activate
    def test_get_backup_reserved_percentage(self):
        add(
            Response(responses.GET, url=f"{ENDPOINT}operation", json=OPERATION_RESPONSE)
        )

    @responses.activate
    def test_get_operation_mode(self):
        add(
            Response(responses.GET, url=f"{ENDPOINT}operation", json=OPERATION_RESPONSE)
        )

    @responses.activate
    def test_get_version(self):
        add(Response(responses.GET, url=f"{ENDPOINT}status", json=STATUS_RESPONSE))
        self.assertEqual(self.powerwall.get_version(), "1.50.1")

    @responses.activate
    def test_detect_and_pin_version(self):
        add(Response(responses.GET, url=f"{ENDPOINT}status", json=STATUS_RESPONSE))
        vers = version.LooseVersion("1.50.1")
        pw = Powerwall(ENDPOINT)
        self.assertEqual(pw.detect_and_pin_version(), vers)
        self.assertEqual(pw._pin_version, vers)

    def test_helpers(self):
        resp = {"a": 1}
        with self.assertRaises(MissingAttributeError):
            assert_attribute(resp, "test")

        with self.assertRaises(MissingAttributeError):
            assert_attribute(resp, "test", "test")

        self.assertEqual(convert_to_kw(2500, -1), 2.5)



import unittest
import json

import requests
import responses
from responses import GET, POST, Response, add

from tesla_powerwall import API, AccessDeniedError, PowerwallUnreachableError, APIError

from tests.unit import ENDPOINT


class TestAPI(unittest.TestCase):
    def setUp(self):
        self.api = API(ENDPOINT)

    def test_parse_endpoint(self):
        test_endpoints = [
            "1.1.1.1",
            "http://1.1.1.1",
            "https://1.1.1.1/api/",
            "https://1.1.1.1/api",
            "https://1.1.1.1/",
        ]
        for test_endpoint in test_endpoints:
            self.assertEqual(self.api._parse_endpoint(test_endpoint), ENDPOINT)

    @responses.activate
    def test_process_response(self):
        res = requests.Response()
        res.request = requests.Request(method="GET", url=f"{ENDPOINT}test").prepare()
        res.status_code = 401
        with self.assertRaises(AccessDeniedError):
            self.api._process_response(res)

        res.status_code = 404
        with self.assertRaises(APIError):
            self.api._process_response(res)

        res.status_code = 502
        with self.assertRaises(APIError):
            self.api._process_response(res)

        res.status_code = 200
        res._content = b'{"error": "test_error"}'
        with self.assertRaises(APIError):
            self.api._process_response(res)

        res._content = b'{invalid_json"'
        with self.assertRaises(APIError):
            self.api._process_response(res)

        res._content = b"{}"
        self.assertEqual(self.api._process_response(res), {})

        res._content = b'{"response": "ok"}'
        self.assertEqual(self.api._process_response(res), {"response": "ok"})

    @responses.activate
    def test_get(self):
        add(Response(GET, url=f"{ENDPOINT}test_get", json={"test_get": True}))

        self.assertEqual(self.api.get("test_get"), {"test_get": True})

    @responses.activate
    def test_post(self):
        def post_callback(request):
            resp_body = {"test_post": True}
            headers = {}
            return (200, headers, json.dumps(resp_body))

        responses.add_callback(
            responses.POST,
            url=f"{ENDPOINT}test_post",
            callback=post_callback,
            content_type="application/json",
        )

        resp = self.api.post("test_post", {"test": True})

        self.assertIsInstance(resp, dict)
        self.assertEqual(resp, {"test_post": True})

    def test_is_authenticated(self):
        api = API(ENDPOINT)
        self.assertEqual(api.is_authenticated(), False)

    def test_url(self):
        self.assertEqual(self.api.url("test"), ENDPOINT + "test")
